<?php
#*****************************************************************************
# 
# Author: Jason Wall
# Definition: The Template module 
#
#*****************************************************************************

class Template {

	var $parser;
	var $tags;
	var $template_text;
	
	/*
	# Constructor function includes Parser and Tag handler functions, and initializes the objects.
	*/
	function Template($templateText) {
  	$this->template_text = $templateText;
		require_once(dirname(__FILE__)."/Parser.php");
		$this->parser = new Parser();
		
  }
  
  # Build the parse tree.
  #  the parse() function takes the template and turns it into a tree of 
  #	 tag/text tokens that will be passed to the handlers later.
  function parse($template_text, $tag_prefix) {
  		$token_tree = array();
      $tokens = $this->parser->parse_text($template_text,$tag_prefix);
      foreach ($tokens as $token) {
          if ($token['type'] == 'container') {
              $sub_tokens = $this->parse($token['block'],$tag_prefix);
              $token['block'] = $sub_tokens;
          }
          array_push($token_tree, $token);
      }
      return $token_tree;
  }
  
  # The build_template is the only other publicly defined function in this module, aside from the new() function.
  #  This function calls the handlers, passing the appropriate context info and parameters, and compiles the built
  #  and processed output.
  function build_template ($parameters = '') {
  		# preprocess pass (normally done to process Includes, but could be used for
  		#  for other things potentially)
			require_once(dirname(__FILE__)."/TagHandlers.php");
			$tags = new TagHandlers($this);
  		
			$preprocess_tree = $this->parse($this->template_text,'PRE');
  		$preprocess_text = $this->call_handlers($parameters, $preprocess_tree, $tags);

  		# normal pass
  		$normal_tree = $this->parse($preprocess_text,'WP');
  		$normal_text = $this->call_handlers($parameters, $normal_tree, $tags);

  		# postprocess pass
  		$postprocess_tree = $this->parse($normal_text, 'POST');
      $postprocess_text = $this->call_handlers($parameters, $postprocess_tree, $tags);
  	

//			echo var_dump($this->token_tree);

  		return $postprocess_text;
  }
  
  function call_handlers ($parameters, $token_tree, $tags)  {
    $parsed_text = '';

  	# loop through each token in the tree, determin its type and process it
  	#  appropriately.

      foreach ($token_tree as $token) {
  			
    		# if the token is a single or block tag, call its handler.
    		if ($token['type'] == "single" || $token['type'] == "container") 
				{
    		
    			# we are dynamically loading our handlers here
    			# using symbolic references and a little perl magic
    			$tagname = $token['tagname'];
					
					if (method_exists($tags,$tagname)) {
						$parsed_text .= $tags->$tagname($token, $parameters);
					} else {
						echo var_dump($token);
						$parsed_text .= "[Error: INVALID Tag '" . $tagname . "']";
					}

    		} elseif ($token['type'] == "text") {
    		  # if the token is a text block, add the text block to the output stringg.
    			$parsed_text .= $token['block'];

    		} else {
    			# if an invalid token type crept into the tree, let us know.
    			$parsed_text .= "[Error: INVALID token type '" . $token['type'] . "']";
    		}
      }
      
  		# return the finished output.
      return $parsed_text;
  }

}

?>


